local addonName, addon = ...
local E = addon:Eve()

local GuidCache = {
	-- [guid] = { cached datas, timestamp }
}

local CACHE_TIMEOUT = 60 -- seconds to store cached values
local Unit = Unit
local ScanningGUID
hooksecurefunc('NotifyInspect', function(unitID)
	ActiveGUID = nil
	local guid = UnitGUID(unitID)
	if guid and not ScanningGUID then
		ScanningGUID = guid
	end
end)

local ActiveGUID

local function ColorGradient(perc, r1, g1, b1, r2, g2, b2)
	if perc >= 1 then
		local r, g, b = r2, g2, b2 -- select(select('#', ...) - 2, ...)
		return r, g, b
	elseif perc <= 0 then
		local r, g, b = r1, g1, b1
		return r, g, b
	end

	--local num = 2 -- select('#', ...) / 3

	--local segment, relperc = math.modf(perc) --*(num-1))
	--local r1, g1, b1, r2, g2, b2 = select((segment*3)+1, ...)
	

	return r1 + (r2 - r1) * perc, g1 + (g2 - g1) * perc, b1 + (b2 - b1) * perc
end

local function ColorDiff(a, b)
	local diff = a - b
	local perc = diff / 30
	
	local r, g, b
	if perc < 0 then -- higher ilevel than us
		perc = perc * -1
		r, g, b = ColorGradient(perc, 1, 1, 0, 0, 1, 0)
	else
		r, g, b = ColorGradient(perc, 1, 1, 0, 1, 0, 0)
	end
	return r, g, b
end

local function FixLink(link)
	-- Attempt to fix an issue with GetDetailedItemLevelInfo and scaling items
	return (link:gsub('(.*item:[^:]*:[^:]*:[^:]*:[^:]*:[^:]*:[^:]*:[^:]*:[^:]*:)[^:]*:(.*):(%d+)(:*)', '%1%3:%2:%3%4'))
end

-- todo: add tooltip line immediately, then update it with their item level when it becomes available
local function DecorateTooltip()
	local guid = UnitGUID(Unit)
	if guid and GuidCache[guid] then
		local averageItemLevel = GuidCache[guid].itemLevel
		local weaponLevel = GuidCache[guid].weaponLevel
		local ourMaxItemLevel, ourEquippedItemLevel = GetAverageItemLevel()
		
		local r, g, b = ColorDiff(ourEquippedItemLevel, averageItemLevel)
		local ourWeaponMain = GetInventoryItemLink('player', 16)
		local ourWeaponOff = GetInventoryItemLink('player', 17)
		local ourWeaponLevel = 0
		if ourWeaponOff and ourWeaponMain then -- dual wielding
			local mainLevel = GetDetailedItemLevelInfo(ourWeaponMain)
			local offLevel = GetDetailedItemLevelInfo(ourWeaponOff)
			if mainLevel > offLevel then
				ourWeaponLevel = mainLevel
			else
				ourWeaponLevel = offLevel
			end
		elseif ourWeaponMain then
			ourWeaponLevel = GetDetailedItemLevelInfo(ourWeaponMain)
		elseif ourWeaponOff then
			ourWeaponLevel = GetDetailedItemLevelInfo(ourWeaponOff)
		end
		local r2, g2, b2 = ColorDiff(ourWeaponLevel, weaponLevel)
		
		
		for i = 3, GameTooltip:NumLines() do -- bail if we've already added a line to this tooltip so we don't add it twice
			local str = _G['GameTooltipTextLeft' .. i]
			local text = str and str:GetText()
			if text == STAT_AVERAGE_ITEM_LEVEL then
				local str2 = _G['GameTooltipTextRight' .. i]
				str2:SetText(format('%.1f |cff%2x%2x%2x(%s)|r', averageItemLevel, r2 * 255, g2 * 255, b2 * 255, weaponLevel))
				str2:SetTextColor(r, g, b)
				str:SetTextColor(r, g, b)
				return
			end
		end
		GameTooltip:AddDoubleLine(STAT_AVERAGE_ITEM_LEVEL, format('%.1f |cff%2x%2x%2x(%s)|r', averageItemLevel, r2 * 255, g2 * 255, b2 * 255, weaponLevel), r, g, b, r, g, b)
		GameTooltip:Show()
	end
end

local function ScanMouseover()
	local self = GameTooltip
	ActiveGUID = nil
	local name, unit = self:GetUnit()
	if unit and UnitIsPlayer(unit) then
		local unitGUID = UnitGUID(unit)
		if unitGUID then
			Unit = unit
			---[[
			if GuidCache[unitGUID] and (GetTime() - GuidCache[unitGUID].timestamp) <= CACHE_TIMEOUT then
				-- show cached data IF timestamp doesn't need refreshing
				DecorateTooltip()
			elseif not InCombatLockdown() and CanInspect(unit) and not UnitIsDeadOrGhost('player') then
				--ClearInspectPlayer() -- todo: this is probably important but it's slowing things down
				GameTooltip:AddDoubleLine(STAT_AVERAGE_ITEM_LEVEL, 'Loading..', 0.6, 0.6, 0.6, 0.6, 0.6, 0.6)
				GameTooltip:Show()
				NotifyInspect(unit)
			end
			--]]
		end
	end
end

GameTooltip:HookScript('OnTooltipSetUnit', ScanMouseover)


local TwoHanders = {
	-- item types that are two handed, as returned as the 4th result from GetItemInfoInstant
	['INVTYPE_RANGED'] = true,
	['INVTYPE_RANGEDRIGHT'] = true,
	['INVTYPE_2HWEAPON'] = true,
}

local InventorySlots = {}
for i = 1, 17 do
	if i ~= 4 then -- ignore shirt, tabard is 19
		tinsert(InventorySlots, i)
	end
end

local ItemQueue = {} -- store a list of slots we failed to fetch data for and repeatedly request links until it works
local ItemCache = {} -- [slot] = itemLink for inspected unit

local function IsArtifact(itemLink)
	return itemLink:find('|cffe6cc80') -- this is probably a horrible way to find whether it's an artifact
end

local function IsCached(itemLink) -- we can't get the correct level of an artifact until all of its relics have been cached
	local cached = true
	local _, itemID, _, relic1, relic2, relic3 = strsplit(':', itemLink)
	if not GetDetailedItemLevelInfo(itemID) then cached = false end
	if IsArtifact(itemLink) then
		if relic1 and relic1 ~= '' and not GetDetailedItemLevelInfo(relic1) then cached = false end
		if relic2 and relic2 ~= '' and not GetDetailedItemLevelInfo(relic2) then cached = false end
		if relic3 and relic3 ~= '' and not GetDetailedItemLevelInfo(relic3) then cached = false end
	end
	return cached
end

local function ProcessItems()
	-- average item level is sum of item levels / 16
	local totalItemLevel = 0
	local itemCount = 0
	for slot, itemLink in pairs(ItemCache) do
		-- if item is an artifact and 
		if slot ~= 16 and slot ~= 17 then
			local ilevel = GetDetailedItemLevelInfo(itemLink)
			totalItemLevel = totalItemLevel + ilevel
		end
		
		itemCount = itemCount + 1
	end
	
	-- handle weapon separately
	-- if we have one 2 hander, count its item level twice
	-- if we have two 1 handers and it's an ARTIFACT then count the highest of the 2 twice
	
	local weaponLevel = 0
	if ItemCache[16] and ItemCache[17] then -- we have 2 weapons
		if IsArtifact(ItemCache[16]) then -- take the higher of the 2 weapons and double it
			local ilevelMain = GetDetailedItemLevelInfo(ItemCache[16])
			local ilevelOff = GetDetailedItemLevelInfo(ItemCache[17])
			if ilevelMain > ilevelOff then
				totalItemLevel = totalItemLevel + (ilevelMain * 2)
				weaponLevel = ilevelMain
			else
				totalItemLevel = totalItemLevel + (ilevelOff * 2)
				weaponLevel = ilevelOff
			end
		else
			local ilevelMain = GetDetailedItemLevelInfo(ItemCache[16])
			local ilevelOff = GetDetailedItemLevelInfo(ItemCache[17])
			totalItemLevel = totalItemLevel + ilevelMain + ilevelOff
			if ilevelMain > ilevelOff then
				weaponLevel = ilevelMain
			else
				weaponLevel = ilevelOff
			end
		end
	elseif ItemCache[16] then -- main hand only
		local _, _, _, weaponType = GetItemInfoInstant(ItemCache[16])
		local ilevelMain = GetDetailedItemLevelInfo(ItemCache[16])
		weaponLevel = ilevelMain
		if TwoHanders[weaponType] then -- 2 handed, count it twice
			totalItemLevel = totalItemLevel + (ilevelMain * 2)
		else
			totalItemLevel = totalItemLevel + ilevelMain
		end
	elseif ItemCache[17] then -- off hand only?
		local ilevelOff = GetDetailedItemLevelInfo(ItemCache[17])
		totalItemLevel = totalItemLevel + ilevelOff
		weaponLevel = ilevelOff
	end
	
	local averageItemLevel = totalItemLevel / 16
	
	
	if GameTooltip:IsVisible() then -- todo: make sure this is currently showing the unit we inspected
		local guid = UnitGUID(Unit)
		if guid and guid == ActiveGUID then
			GuidCache[ActiveGUID] = {timestamp = GetTime(), itemLevel = averageItemLevel, weaponLevel = weaponLevel}
			DecorateTooltip()
		end
	end
end

local f = CreateFrame('frame')
f:Hide()
f:SetScript('OnUpdate', function(self, elapsed)
	-- todo: this will repeat indefinitely if we think an item hasn't been cached, so we should also time out at some point
	-- finish populating our item cache if we were missing info
	local finished = true
	for slot in pairs(ItemQueue) do
		local itemLink = GetInventoryItemLink(Unit, slot)
		if itemLink then
			local isCached = IsCached(itemLink)
			if isCached then
				ItemCache[slot] = FixLink(itemLink)
				ItemQueue[slot] = nil
			else
				finished = false
			end
		else
			finished = false
		end
	end
	if finished then
		self:Hide()
		-- do final callback with our datas
		ProcessItems()
	end
end)

-- to get the correct item level for artifacts:
-- the artifact itself has to be cached
-- each relic in the artifact has to be cached

function E:INSPECT_READY(inspectedGUID)
	-- fires when information about our inspect target becomes available
	-- i don't know what happens when you try to inspect a new person before this event fires for the previous inspect target
	local scannedGUID = ScanningGUID
	ScanningGUID = nil
	if GameTooltip:IsVisible() and not ActiveGUID then
		local _, unit = GameTooltip:GetUnit()
		if unit and UnitIsPlayer(Unit) then
			if UnitGUID(Unit) == inspectedGUID then -- scannedGUID then
				local guid = UnitGUID(Unit)
				-- iterate over inspected unit's slots, add up their average item level
				-- if we have an artifact in the offhand, ignore whichever weapon slot is lower (750)
				wipe(ItemQueue)
				wipe(ItemCache)
				local numItems = 0
				local numTextures = 0
				local numIDs = 0
				
				local missingData = false
				for i, slot in pairs(InventorySlots) do
					local texture = GetInventoryItemTexture(Unit, slot)
					if texture then
						numTextures = numTextures + 1
					end
					-- this seems to return data even when we can't get the link yet
					if texture then --GetInventoryItemID(Unit, slot) then
						numIDs = numIDs + 1
						local itemLink = GetInventoryItemLink(Unit, slot)
						if itemLink then
							local isCached = IsCached(itemLink)
							if isCached then
								ItemCache[slot] = FixLink(itemLink)
								numItems = numItems + 1
							else
								ItemQueue[slot] = true
								missingData = true
							end
						else
							-- put the slot in our queue to check OnUpdate until it works
							ItemQueue[slot] = true
							missingData = true
						end
					end
				end
				
				--if missingData then
					f:Show()
				--end
				ActiveGUID = guid
		
			else
				ScanMouseover()
			end
		end
	end
end
